#pragma once

namespace a8
{
    namespace mysql
    {
        class Query;
        class Connection;
    }
}

struct OrderInfo
{
    std::string seqid;
    std::string orderid;
    std::string accountid;
    int itemid = 0;
    int price = 0;
    int customprice = 0;
    int serverid = 0;
    std::string roleid;
    std::string rolename;
    std::string sp_accountid;
    std::string channel;
    std::string itemname;
    std::string itemmemo;
    std::string sp_orderid;
    time_t lastchecktime = 0;
};

struct AddOrderResult
{
    bool isok = false;
    OrderInfo orderinfo;
};

struct AddOrderNode;
struct PayNotifyNode;
struct ReissueNode;
class OrderMgr : public a8::Singleton<OrderMgr>
{

 private:
    OrderMgr() {};
    friend class a8::Singleton<OrderMgr>;

 public:
    void Init();
    void UnInit();

    void Update();
    bool HasTask();

    int AddOrder(const std::string& seqid,
                 const std::string& accountid,
                 const std::string& roleid,
                 const std::string& rolename,
                 const std::string& channel,
                 const std::string& sp_accountid, int serverid,
                 int productid, int price,
                 const std::string& sp_orderid);
    bool WaitForAddOrderFinished(int seqid,
                                 AddOrderResult& addresult,
                                 int seconds);

    int AddPayNotify(
                     int pay_type,
                     const std::string& accountid,
                     const std::string& roleid,
                     const std::string& rolename,
                     int serverid,
                     int itemid,
                     const std::string& itemid_str,
                     const std::string& orderid,
                     const std::string& sp_orderid,
                     double fee,
                     int payresult);
    bool WaitForPayNotifyFinished(int seqid, int& result, int seconds);

    int AddReissue(const std::string& orderid);
    bool WaitForReissueFinished(int seqid, int& result, int seconds);

    //new
    bool AddNewOrder(const std::string& accountid,
                     int serverid,
                     const std::string& roleid,
                     const std::string& rolename,
                     int itemid, int price,
                     const std::string& sp_orderid,
                     OrderInfo& orderinfo);
    void AddNewPayNotify(
                        int pay_type,
                        const std::string& orderid,
                        int payresult,
                        int& result);
    void G2PPayNotifyACK(a8::XParams& param, a8::XObject& data);

 private:
    void ProcessOrderMsg();
    void ProcessNotifyMsg();
    void ProcessReissueMsg();
    bool ProcAddOrder(OrderInfo& orderinfo);
    int ProcOrderNotify(PayNotifyNode* node);
        int ContinueSubscriptions(PayNotifyNode* node);
    int ProcIosPayOrderNotify(PayNotifyNode* node);
    void ProcReissue(const std::string& orderid);
    void UpdatePedingOrders();
    void LoadPedingOrders();
    std::string GenOrderId(int itemid, int price = 0);
    long GenSeqId();

    #if 0
    void _SSMPing(MsgHdr& hdr, const SSMPing& msg);

    void _D2PPayNotifyACK(MsgHdr& hdr, const D2PPayNotifyACK& msg);
    #endif

    //new
    int ProcOrderNotifyNew(int pay_type,
                           const std::string& orderid,
                           int payresult);
    void SendGMMsg(int serverid,
                   const std::string& orderid,
                   const std::string& roleid,
                   int itemid, int price,
                   const std::string& rolename);

 private:
    a8::mysql::Connection* mysql_conn_ = nullptr;
    a8::mysql::Query* query_ = nullptr;
    a8::tick_t last_ping_db_tick_ = 0;

    std::mutex *add_order_mutex_ = nullptr;
    AddOrderNode *top_node_ = nullptr;
    AddOrderNode *bot_node_ = nullptr;
    AddOrderNode *work_node_ = nullptr;

    std::mutex *pay_notify_mutex_ = nullptr;
    PayNotifyNode *pay_notify_top_node_ = nullptr;
    PayNotifyNode *pay_notify_bot_node_ = nullptr;
    PayNotifyNode *pay_notify_work_node_ = nullptr;

    std::mutex *reissue_mutex_ = nullptr;
    ReissueNode *reissue_top_node_ = nullptr;
    ReissueNode *reissue_bot_node_ = nullptr;
    ReissueNode *reissue_work_node_ = nullptr;

    int last_gen_order_time_ = 0;
    int sub_orderid_ = 0;

    std::mutex *seq_mutex_ = nullptr;
    volatile int current_seqid_ = 0;
    std::mutex *add_order_result_mutex_ = nullptr;
    std::map<int, AddOrderResult> add_order_results_;

    std::mutex *pay_notify_result_mutex_;
    std::map<int, int> pay_notify_results_;

    std::map<std::string, OrderInfo> peding_orders_;
};
